//mod index;
use crate::index::SYMBOL;
use crate::index::SYMBOL_INDEX;
use crate::atomic_mass::ATOMIC_MASS;

pub fn nucleons_nuclide(x: &(usize, usize))-> usize{
    SYMBOL_INDEX[x.0-1].0 -(SYMBOL_INDEX[x.0 -1].1-x.0) + x.1 
 }

 pub fn rand()->u64{
   let mut x: u64 = 0;
  let k = unsafe{core::arch::x86_64::_rdrand64_step(&mut x)};
x
}


 pub struct Nuclide{
     nuclide: usize,
}


impl Clone for Nuclide{
  fn clone(&self)->Nuclide{
  Nuclide{nuclide: self.nuclide.clone()}
  }
}

impl Nuclide{

  pub fn new(x: &str, isotope: usize)-> Result<Nuclide,String >{
 
         match SYMBOL.iter().position(|y| y ==&x){
          Some(x)=> if isotope >= SYMBOL_INDEX[x].1 &&  isotope <= SYMBOL_INDEX[x].2 {
                       return Ok(
                     Nuclide{nuclide: SYMBOL_INDEX[x].0+isotope-SYMBOL_INDEX[x].1}
                     )
                   }
                   else{
                   return Err("Not a known isotope".to_string())
                   }
         None=> return Err("Not a known element".to_string())
       }
       
       }
 
  pub fn assign(x: usize)->Nuclide{
      Nuclide{nuclide: x}
  }
 
  pub  fn change(&mut self, x: usize){
          self.nuclide = x
     }
 
  pub  fn nuclide_index(&self)->usize{
          self.nuclide.clone()
     }
 
 
  pub  fn atomic_num(&self)->usize{
          let mut count: usize = 0;
          for i in SYMBOL_INDEX{
  
           if i.0 > self.nuclide{
              break;
           }
         count+=1;
         }
        count
       }

pub  fn isotope(&self)->(usize, usize){
          let element = self.atomic_num();
          (element, 
          (self.nuclide -SYMBOL_INDEX[element-1].0)+SYMBOL_INDEX[element-1].1
          )
       }
 
  pub  fn identity(&self)-> String{
          let iso = self.isotope();
          SYMBOL[iso.0-1].to_owned() + "-"+ &iso.1.to_string()
       }
 
  pub  fn proton_neutron(&self)->(usize, usize){
          (self.isotope().0,  self.isotope().1 - self.isotope().0)
       }
       
  pub  fn amu(&self)->f64{
          ATOMIC_MASS[self.nuclide]
       }
 
  pub  fn am_kg(&self)->f64{
          self.amu()*1.6605390666E-27
       }
       
  pub  fn mass_deficit_amu(&self)->f64{
          let nucleons = self.proton_neutron();
          (1.007276466621*nucleons.0 as f64 + 5.48579909065E-4*nucleons.0 as f64
           + 1.00866491588*nucleons.1 as f64)-self.amu()
       }
 
  pub  fn mass_deficit(&self)->f64{
       self.mass_deficit_amu()* 1.6605390666E-27
       }
       
  //Result as electron volts, implement Myers Swiatecki semi-empirical equation
  pub  fn binding_energy(&self)->f64{
          self.mass_deficit_amu()*931.49410242
       }
       
      
}
